"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

const withPerformanceTracerImpl = async (pool, methodName, methodImpl, ...args) => {
  const segment = pool.performanceTracer != null ? pool.performanceTracer.getSegment() : null;
  const subSegment = segment ? segment.addNewSubsegment(methodName) : null;
  const maybeReadModelName = args.length > 1 && args[1] != null && args[1].constructor === String ? String(args[1]) : args.length === 1 && args[0] != null && args[0].constructor === String ? String(args[0]) : null;

  if (subSegment != null) {
    if (maybeReadModelName != null) {
      subSegment.addAnnotation('readModelName', maybeReadModelName);
    }

    subSegment.addAnnotation('origin', `resolve:readmodel:${methodName}`);
  }

  try {
    return await methodImpl(...args);
  } catch (error) {
    if (subSegment != null) {
      subSegment.addError(error);
    }

    throw error;
  } finally {
    if (subSegment != null) {
      subSegment.close();
    }
  }
};

const withPerformanceTracer = (pool, methodName, methodImpl) => {
  return withPerformanceTracerImpl.bind(null, pool, methodName, methodImpl);
};

var _default = withPerformanceTracer;
exports.default = _default;
//# sourceMappingURL=with-performance-tracer.js.map